@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2012 The PHP Group                                |
+   | Copyright (c) 1997-2015 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -16,15 +16,17 @@
    +----------------------------------------------------------------------+
  */
 
-/* $Id: mail.c 321634 2012-01-01 13:15:04Z felipe $ */
+/* $Id$ */
 
 #include <stdlib.h>
 #include <ctype.h>
 #include <stdio.h>
+#include <time.h>
 #include "php.h"
 #include "ext/standard/info.h"
 #include "ext/standard/php_string.h"
 #include "ext/standard/basic_functions.h"
+#include "ext/date/php_date.h"
 
 #if HAVE_SYSEXITS_H
 #include <sysexits.h>
@@ -39,6 +41,7 @@
 #endif
 #endif
 
+#include "php_syslog.h"
 #include "php_mail.h"
 #include "php_ini.h"
 #include "php_string.h"
@@ -189,6 +192,74 @@ PHP_FUNCTION(mail)
 }
 /* }}} */
 
+
+void php_mail_log_crlf_to_spaces(char *message) {
+	/* Find all instances of carriage returns or line feeds and
+	 * replace them with spaces. Thus, a log line is always one line
+	 * long
+	 */
+	char *p = message;
+	while ((p = strpbrk(p, "\r\n"))) {
+		*p = ' ';
+	}
+}
+
+void php_mail_log_to_syslog(char *message) {
+	/* Write 'message' to syslog. */
+#ifdef HAVE_SYSLOG_H
+	php_syslog(LOG_NOTICE, "%s", message);
+#endif
+}
+
+
+void php_mail_log_to_file(char *filename, char *message, size_t message_size TSRMLS_DC) {
+	/* Write 'message' to the given file. */
+	uint flags = IGNORE_URL_WIN | REPORT_ERRORS | STREAM_DISABLE_OPEN_BASEDIR;
+	php_stream *stream = php_stream_open_wrapper(filename, "a", flags, NULL);
+	if (stream) {
+		php_stream_write(stream, message, message_size);
+		php_stream_close(stream);
+	}
+}
+
+
+static int php_mail_detect_multiple_crlf(char *hdr) {
+	/* This function detects multiple/malformed multiple newlines. */
+
+	if (!hdr || !strlen(hdr)) {
+		return 0;
+	}
+
+	/* Should not have any newlines at the beginning. */
+	/* RFC 2822 2.2. Header Fields */
+	if (*hdr < 33 || *hdr > 126 || *hdr == ':') {
+		return 1;
+	}
+
+	while(*hdr) {
+		if (*hdr == '\r') {
+			if (*(hdr+1) == '\0' || *(hdr+1) == '\r' || (*(hdr+1) == '\n' && (*(hdr+2) == '\0' || *(hdr+2) == '\n' || *(hdr+2) == '\r'))) {
+				/* Malformed or multiple newlines. */
+				return 1;
+			} else {
+				hdr += 2;
+			}
+		} else if (*hdr == '\n') {
+			if (*(hdr+1) == '\0' || *(hdr+1) == '\r' || *(hdr+1) == '\n') {
+				/* Malformed or multiple newlines. */
+				return 1;
+			} else {
+				hdr += 2;
+			}
+		} else {
+			hdr++;
+		}
+	}
+
+	return 0;
+}
+
+
 /* {{{ php_mail
  */
 PHPAPI int php_mail(char *to, char *subject, char *message, char *headers, char *extra_cmd TSRMLS_DC)
@@ -214,23 +285,35 @@ PHPAPI int php_mail(char *to, char *subject, char *message, char *headers, char
 	return val;	\
 
 	if (mail_log && *mail_log) {
-		char *tmp;
-		int l = spprintf(&tmp, 0, "mail() on [%s:%d]: To: %s -- Headers: %s\n", zend_get_executed_filename(TSRMLS_C), zend_get_executed_lineno(TSRMLS_C), to, hdr ? hdr : "");
-		php_stream *stream = php_stream_open_wrapper(mail_log, "a", IGNORE_URL_WIN | REPORT_ERRORS | STREAM_DISABLE_OPEN_BASEDIR, NULL);
+		char *tmp, *date_str;
+		time_t curtime;
+		int l;
 
-		if (hdr) { /* find all \r\n instances and replace them with spaces, so a log line is always one line long */ 
-			char *p = tmp;
-			while ((p = strpbrk(p, "\r\n"))) {
-				*p = ' ';
-			}
+		time(&curtime);
+		date_str = php_format_date("d-M-Y H:i:s e", 13, curtime, 1 TSRMLS_CC);
+
+		l = spprintf(&tmp, 0, "[%s] mail() on [%s:%d]: To: %s -- Headers: %s\n", date_str, zend_get_executed_filename(TSRMLS_C), zend_get_executed_lineno(TSRMLS_C), to, hdr ? hdr : "");
+
+		efree(date_str);
+
+		if (hdr) {
+			php_mail_log_crlf_to_spaces(tmp);
+		}
+
+		if (!strcmp(mail_log, "syslog")) {
+			/* Drop the final space when logging to syslog. */
+			tmp[l - 1] = 0;
+			php_mail_log_to_syslog(tmp);
+		}
+		else {
+			/* Convert the final space to a newline when logging to file. */
 			tmp[l - 1] = '\n';
+			php_mail_log_to_file(mail_log, tmp, l TSRMLS_CC);
 		}
-		if (stream) {
-			php_stream_write(stream, tmp, l);
-			php_stream_close(stream);
-		}
+
 		efree(tmp);
 	}
+
 	if (PG(mail_x_header)) {
 		const char *tmp = zend_get_executed_filename(TSRMLS_C);
 		char *f;
@@ -241,11 +324,16 @@ PHPAPI int php_mail(char *to, char *subject, char *message, char *headers, char
 		if (headers != NULL) {
 			spprintf(&hdr, 0, "X-PHP-Originating-Script: %ld:%s\n%s", php_getuid(TSRMLS_C), f, headers);
 		} else {
-			spprintf(&hdr, 0, "X-PHP-Originating-Script: %ld:%s\n", php_getuid(TSRMLS_C), f);
+			spprintf(&hdr, 0, "X-PHP-Originating-Script: %ld:%s", php_getuid(TSRMLS_C), f);
 		}
 		efree(f);
 	}
 
+	if (hdr && php_mail_detect_multiple_crlf(hdr)) {
+		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Multiple or malformed newlines found in additional_header");
+		MAIL_RET(0);
+	}
+
 	if (!sendmail_path) {
 #if (defined PHP_WIN32 || defined NETWARE)
 		/* handle old style win smtp sending */
@@ -283,7 +371,7 @@ PHPAPI int php_mail(char *to, char *subject, char *message, char *headers, char
 	sendmail = popen_ex(sendmail_cmd, "wb", NULL, NULL TSRMLS_CC);
 #else
 	/* Since popen() doesn't indicate if the internal fork() doesn't work
-	 * (e.g. the shell can't be executed) we explicitely set it to 0 to be
+	 * (e.g. the shell can't be executed) we explicitly set it to 0 to be
 	 * sure we don't catch any older errno value. */
 	errno = 0;
 	sendmail = popen(sendmail_cmd, "w");
